'''
Created on 7 Apr 2016

@author: steve
'''
class DNA(object):
    dna_alphabet = set("AGCTN")
    def __init__(self, sequence):
        self.sequence = sequence.upper()
    def __len__(self):
        return len(self.sequence)
    def __getitem__(self, key):
        return self.sequence[key]
    
    def __hash__(self):
        return hash((self.sequence))

    def __repr__(self):
        return self.sequence
    
    def __eq__(self, other):
        return self.sequence == other.sequence
    
    def complement(self):
        """Provides the complement in the 5' - 3' direction

        Assumption: reference consists of A, G, C, T only

        complement(str) --> str
        """
        d = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C'}
        return (''.join(d[c] if c in d else c for c in reversed(self.sequence))) 
    
    def is_dna(self):
        """
        True if all nucleotides are from dna_alphabet
        """
        not_dna = set(self) - DNA.dna_alphabet
        return not not_dna


'''
Created on 30 Mar 2016

@author: steve
'''

"""
Helper functions for the analysis Module
"""

def single_file_output(in_file):
    """
    Parse in_file path to produce a name 
    for the output file
    
    In: path/to/in_file.fa
    Out: in_file
    """
    return in_file.split('/')[-1].split('.')[-2]

def rep_file_output(in_file_1, in_file_2):
    """
    Parse replicate in_file paths to produce a name 
    for the output file
    
    In: path/to/in_file_1.fa path/to/in_file_2.fa
    Out: in_file_1_in_file_2
    """
    return "{0}_{1}".format(single_file_output(in_file_1),
                            single_file_output(in_file_2))
    
def ref_seq_nt_output(in_seq_name, in_ref_name, nt, ext):
    """
    Generates a compound file name from a single_file_output, 
    rep_file_output, and nt with the correct enxtension
    
    Out: in_ref_name_in_seq_name_nt.ext
    """
    return "{0}_{1}_{2}.{3}".format(in_ref_name,
                                    in_seq_name,
                                    str(nt),
                                    ext)
def ref_seq_output(in_seq_name, in_ref_name, ext):
    """
    Generates a compound file name from a single_file_output, 
    rep_file_output, and nt with the correct enxtension
    
    Out: in_ref_name_in_seq_name_nt.ext
    """
    return "{0}_{1}.{2}".format(in_ref_name,
                                    in_seq_name,
                                    ext)
def header_output(header):
    """
    Fill in
    """
    
    return header[1:]

def cdp_file_output(in_seq_name1, in_seq_name2, ref, nt, ext):
    """
    Fill in
    """
    return "{0}_{1}_{2}_{3}.{4}".format(in_seq_name1,
                                    in_seq_name2,
                                    ref,
                                    nt,
                                    ext)


'''
Created on 1 Apr 2016

@author: steve
'''
"""
A class for reference sequences  - stores header:seq pairs in a dictionary

{header:seq}
"""
import time
from dna import DNA
from termcolor import colored

class Ref_Seq(object):
    def __init__(self):
        self._internal_dict = {}
    
    def __setitem__(self, header, sequence):
        self._internal_dict[header]=sequence
    
    def __getitem__(self, header):
        return self._internal_dict[header]
    
    def __iter__(self):
        return self._internal_dict.iteritems()
    
    def __len__(self):
        return len(self._internal_dict)

    def headers(self):
        return self._internal_dict.keys()

    def sequences(self):
        return self._internal_dict.values()    
    

    def load_ref_file(self, ref_file):
        """
        Load ref file in fasta format
    
        Product ref_dict --> header:sequence
        """
        print colored("\n-----------------LOADING REFERENCE----------------",'green')
        start = time.clock()
        #ref_dict = {}
        ref_count = 0
        loaded_ref = open(ref_file, 'rU')
        full_len_seq = ''
        key = ''
        first_header = True
        for line in loaded_ref:
            if line[0] == '>' and full_len_seq == '':
                key = line.strip()
                ref_count += 1
                if first_header:
                    first_header = False
            elif line[0] == '>' and full_len_seq != '':
                self._internal_dict.update({key: DNA(full_len_seq)})
                key = line.strip()
                full_len_seq = ''
                ref_count += 1
            elif line[0] == '' and full_len_seq != '':
                self._internal_dict.update({key: DNA(full_len_seq)})
                key = line.strip()
                full_len_seq = ''
            elif line[0] == '':
                pass
            else:
                full_len_seq += line.strip().upper()
    
        self._internal_dict.update({key: DNA(full_len_seq)})
    
        print '\n----{0} reference sequences loaded for alignment----'\
            .format(ref_count)
        if len(self._internal_dict) ==1:
            print "\n{0} length = {1} bp".format(ref_file.split('/')[-1],
                                             len(full_len_seq))
        print "\nReference sequence loading time = "\
         + str((time.clock() - start)) + " seconds\n"



'''
Created on 22 Jan 2016

@author: steve
'''
#Graph module
from pylab import * #@UnusedWildImport
import matplotlib.pyplot as plt # @Reimport


def den_plot(x_ref, 
        y_fwd_smoothed, y_rvs_smoothed, nt, fileFig, 
        fileName, onscreen, x_label, plot_ylim, pub = False):
    """
    TODO: add y_lim adjustment
    Single sRNA length den map plot:
    x-axis - reference
    y-axis - RPMR
    21,22,24 nt plots in default colours
    """
    plt.plot(x_ref,y_fwd_smoothed, color=nt_colour(nt), 
             label='{0} nt'.format(nt))
    plt.plot(x_ref,y_rvs_smoothed, color=nt_colour(nt))
    axhline(y=0)
    if pub:
        plt.tick_params(
        axis='both',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom='on',      # ticks along the bottom edge are off
        top='off',
        right='on',
        left='on',         # ticks along the top edge are off
        labelbottom='off',
        labelleft = 'off',
        labelright = 'off',
        labelsize=15) # labels along the bottom edge are off
        clear_frame() 
    else:
        xlabel(x_label)
        ylabel('Reads per million reads')
        plt.legend(loc='best',fancybox=True, framealpha=0.5)
    if plot_ylim !=0:
        ylim(-plot_ylim, plot_ylim)
    fig1 = plt.gcf()
    if onscreen:
        plt.show()
    if fileFig:
        fig1.savefig(fileName, format='pdf')
    plt.close(fig1)


def den_multi_plot_3(x_ref, 
        y_fwd_smoothed_21, y_rvs_smoothed_21, 
        y_fwd_smoothed_22, y_rvs_smoothed_22, 
        y_fwd_smoothed_24, y_rvs_smoothed_24, fileFig,
        fileName, onscreen, x_label, plot_y_lim,
        pub = False):
    
    """
    21,22 and 24 nt sRNA length den map plot:
    x-axis - reference
    y-axis - RPMR
    21,22,24 nt plots in default colours
    """

    plt.plot(x_ref,y_fwd_smoothed_21, color='#00CC00',label='21 nt')
    plt.plot(x_ref,y_rvs_smoothed_21, color='#00CC00')
    plt.plot(x_ref,y_fwd_smoothed_22, color='#FF3399', label='22 nt')
    plt.plot(x_ref,y_rvs_smoothed_22, color='#FF3399')
    plt.plot(x_ref,y_fwd_smoothed_24, color='#3333FF', label='24 nt')
    plt.plot(x_ref,y_rvs_smoothed_24, color='#3333FF')
    axhline(y=0)
    if pub:
        plt.tick_params(
            axis='both',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            bottom='on',      # ticks along the bottom edge are off
            top='off',
            right='on',
            left='on',         # ticks along the top edge are off
            labelbottom='off',
            labelleft = 'off',
            labelright = 'off',
            labelsize=15) # labels along the bottom edge are off
        clear_frame() 

    else:     #no_publication
        xlabel(x_label)
        ylabel('Reads per million reads')
        plt.rc('font',family='Arial')
        plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
           ncol=3, mode="expand", borderaxespad=0., fontsize=12)   
    if plot_y_lim !=0:
        ylim(-plot_y_lim, plot_y_lim)
    fig1 = plt.gcf()
    
    if onscreen:
        plt.show()
    if fileFig:
        fig1.savefig(fileName, format='pdf')
    plt.close(fig1)


def cdp_plot(counts_by_ref, seq1, seq2, nt, onscreen, fileFig, fileName, pub):
    """
    Scatter plot of alignments to references
    x-axis: seq1 reads aligned to reference
    y-axis: seq2 reads aligned to reference
    """
    results_list=[] #list of resutls
    for counts in counts_by_ref.itervalues():
        results_list.append((counts[0]+0.01, counts[1]+0.01))
        #hack that allows zero values to be plotted on a log scale
    results_list=sorted(results_list)
  
    _max=max(results_list[-1][0], results_list[-1][1]) #sets up max x and y scale values
    _max+=float(_max/2)
    
    plt.scatter(*zip(*results_list), 
                s=10, 
                color = nt_colour(nt) , 
                marker = 'o',
                label = "{0} nt".format(nt))
     
    arrow(0.1,0.1,_max,_max, color = 'r')
    xscale('log')
    yscale('log')
    xlim(0.1,_max)
    ylim(0.1,_max)
    if pub:
        plt.tick_params(
            axis='both',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            bottom='on',      # ticks along the bottom edge are off
            top='off',
            right='on',
            left='on',         # ticks along the top edge are off
            labelbottom='off',
            labelleft = 'off',
            labelright = 'off',
            labelsize=15) # labels along the bottom edge are off
        clear_frame() 
    else:
        plt.legend(loc='upper left',fancybox=True, framealpha=0.5)
        xlabel(seq1)
        ylabel(seq2)
    fig1 = plt.gcf()
    if onscreen:
        plt.show()
    if fileFig:
        fig1.savefig(fileName, format='pdf')
    plt.close(fig1)

def clear_frame(ax=None):
    """
    Removes frame for publishing plots
    """ 
    if ax is None: 
        ax = plt.gca() 
    ax.xaxis.set_visible(True) 
    ax.yaxis.set_visible(True) 
    for spine in ax.spines.itervalues(): 
        spine.set_visible(False) 

def nt_colour(nt):
    """
    Set default colours for 21, 22 and 24 nt sRNAs
    """
    if nt==21:
        col = '#00CC00'
    elif nt ==22:
        col = '#FF3399'
    elif nt ==24:
        col = '#3333FF'
    else:
        col = 'black'
    return col










'''CDP analysis class - for calculation of reads aligning to a reference
sequence as (x,y) coordinates for two sets of sequence files

Created on 25 Feb 2016

@author: steve
'''
from termcolor import colored
from ref_seq import Ref_Seq
import write_to_file as wtf
import analysis_helper as ah
import plot_reads as pr
from dna import DNA
from multiprocessing import Process, JoinableQueue, Manager
#TODO: sort a no csv option
def CDP_shared(seq_1, seq_2, seq_name_1, seq_name_2, ref_file, nt,fileFig, 
               fileName, min_read_size, max_read_size, min_read_no, onscreen,
               no_csv, pub, cores):

    """
    Refactored CDP code shared between CDP and avCDP
    """
    workers = cores
    work_queue = JoinableQueue()
    # done_queue = Queue()
    processes = []
    mgr=Manager()
    count = 0
    counts_by_ref=mgr.dict() #header:(count1, count2)
    refs=Ref_Seq()
    refs.load_ref_file(ref_file)
    print colored("------------------ALIGNING READS------------------\n",'green')
    for header,seq in refs:
        work_queue.put((header,seq,)) 
        count+=1
        if count%10000==0:
            for w in xrange(workers):
                p = Process(target=CDP_worker, args=(work_queue, counts_by_ref, 
                                                     seq_1, 
                                                     seq_2,
                                                     nt))
                p.start()
                processes.append(p)       

            for p in processes:
                p.join()
    for w in xrange(workers):
        p = Process(target=CDP_worker, args=(work_queue, counts_by_ref, 
                                             seq_1, 
                                             seq_2, 
                                             nt))
        p.start()
        processes.append(p)       

    for p in processes:
        p.join()
    if len(counts_by_ref)==0: 
        print "\nNo reads aligned to any reference sequence. \
        Output files not generated\n"
    else:
        CDP_output(counts_by_ref.copy(), fileFig, fileName, onscreen, no_csv, seq_name_1, 
               seq_name_2, ref_file, nt, pub)   


def CDP_worker(work_queue, counts_by_ref, seq_1, seq_2, nt):
    """
    Worker process - get ref from work queue, aligns reads from seq_1 and seq_2,
    and adds as (x,y) coords to counts_by_ref if there are alignments.  
    """
    try:
        while not work_queue.empty():
            both_aligned = CDP_single(work_queue.get(), seq_1, seq_2, nt)
            if both_aligned is not None:
                counts_by_ref[both_aligned[0]]=(both_aligned[1],both_aligned[2])
    except Exception, e:
        print e
    return True  


def CDP_single(single_ref, seq_1, seq_2, nt):
    """
    Count for boths seqs aligned to single ref
    """

    single_alignment_1 = count_align_reads_to_seq(seq_1, 
                                                        single_ref[1], nt)
    single_alignment_2 = count_align_reads_to_seq(seq_2, 
                                                        single_ref[1], nt)        
    if single_alignment_1 != 0 or single_alignment_2 !=0:
        return (single_ref[0], single_alignment_1, single_alignment_2,)


def CDP_split_shared(seq_1, seq_2, seq_name_1, seq_name_2, ref_file, 
                     nt, fileFig, fileName,min_read_size, max_read_size, 
                     min_read_no, onscreen, no_csv, pub, cores): 
    """
    Refactored CDP code shared between CDP_split and avCDP_split
    """
       
    workers = cores
    work_queue = JoinableQueue()
    processes = []
    mgr=Manager()
    count = 0
    
    refs=Ref_Seq()
    refs.load_ref_file(ref_file)
    
    alignment_dict_1=mgr.dict() #header:aligned_sRNAs
    alignment_dict_2=mgr.dict()
    
    print colored("------------------ALIGNING READS------------------\n",'green') 
    for header,seq in refs:
        work_queue.put((header,seq,)) 
        count+=1
        if count%10000==0:
            for w in xrange(workers):
                p = Process(target=split_CDP_worker, 
                            args=(work_queue, 
                                  alignment_dict_1, 
                                  alignment_dict_2, 
                                  seq_1, 
                                  seq_2,
                                  nt))
                p.start()
                processes.append(p)       

            for p in processes:
                p.join()
    for w in xrange(workers):
        p = Process(target=split_CDP_worker, 
                    args=(work_queue, 
                          alignment_dict_1, 
                          alignment_dict_2, 
                          seq_1, 
                          seq_2, 
                          nt))
        p.start()
        processes.append(p)       

    for p in processes:
        p.join()
       
    alignment_dict_1=alignment_dict_1.copy()
    alignment_dict_2=alignment_dict_2.copy()     

    times_align_1=times_read_aligns(alignment_dict_1)
    times_align_2=times_read_aligns(alignment_dict_2)
    
    header_split_count_1=split_reads_for_header(alignment_dict_1, 
                                                    times_align_1, 
                                                    seq_1)
    header_split_count_2=split_reads_for_header(alignment_dict_2, 
                                                    times_align_2, 
                                                    seq_2)  
    
    
    counts_by_ref = header_x_y_counts(header_split_count_1, 
                                          header_split_count_2, 
                                          refs)  
    if len(counts_by_ref)==0: 
        print "\nNo reads aligned to any reference sequence. \
        Output files not generated\n"
    else:
        CDP_output(counts_by_ref, fileFig, fileName, onscreen, no_csv, seq_name_1, 
               seq_name_2, ref_file, nt, pub)
    

def split_CDP_worker(work_queue, alignment_dict_1, alignment_dict_2, seq_1, 
                     seq_2, nt):
    #calc aligned sRNAs for each header, duplicate if necessary
    
    try:
        while not work_queue.empty():
            single_ref=work_queue.get()
            aligned_reads_1 = dict_align_reads_to_seq_split(seq_1, 
                                                            single_ref[1], nt)
            aligned_reads_2 = dict_align_reads_to_seq_split(seq_2, 
                                                            single_ref[1], nt)
            if len(aligned_reads_1) !=0: #FIX
                alignment_dict_1[single_ref[0]] = aligned_reads_1
            if len(aligned_reads_2) !=0: #FIX
                alignment_dict_2[single_ref[0]] = aligned_reads_2
    except Exception, e:
        print e
    return True    
    

def count_align_reads_to_seq(seq_dict, ref, sRNA_length):
    """
    For non-split alignments!
    
    Return mapped reads for a single ref_seq
    pos is 5' end of read relative to 5' end of fwd strand

    returns an integer
    """
    #start = time.clock()
    
    aligned_count = 0 #number of reads aligned

    count_start = 0
    # indv_seq_align_count = 0

    ref_complement = ref.complement()

    while count_start < (len(ref) - (sRNA_length - 1)):
        query_seq_fwd = DNA(ref[count_start:(count_start + sRNA_length)])
        query_seq_rvs = DNA(ref_complement[count_start:(count_start + sRNA_length)])
        
        if query_seq_fwd in seq_dict:
            aligned_count += seq_dict[query_seq_fwd]
        if query_seq_rvs in seq_dict:
            aligned_count += seq_dict[query_seq_rvs]
        count_start += 1

    return aligned_count


def list_align_reads_to_seq_split(seq_dict, ref, sRNA_length):
    #TODO: this is wrong
    """
    Return mapped reads for a single ref_seq
    pos is 5' end of read relative to 5' end of fwd strand

    returns an integer
    alignment_list --> [sRNA,sRNA,]
    
    """
    count_start = 0
    ref_complement = ref.complement()
    alignment_list = [] #aligned sRNAs
    while count_start < (len(ref) - (sRNA_length - 1)):
        query_seq_fwd = DNA(ref[count_start:(count_start + sRNA_length)])
        query_seq_rvs = DNA(ref_complement[count_start:(count_start + sRNA_length)])
        if query_seq_fwd in seq_dict:
            alignment_list.append(query_seq_fwd)
        if query_seq_rvs in seq_dict:
            alignment_list.append(query_seq_rvs)
        count_start += 1  
    return alignment_list 

def dict_align_reads_to_seq_split(seq_dict, ref, sRNA_length):
    """
    Returns a dictionary with the number of times a read aligns to single
    reference sequence
    Out --> {read:times_aligned}
    
    """
    count_start = 0
    ref_complement = ref.complement()
    split_alignment_dict = {} #aligned sRNAs
    while count_start < (len(ref) - (sRNA_length - 1)):
        query_seq_fwd = DNA(ref[count_start:(count_start + sRNA_length)])
        query_seq_rvs = DNA(ref_complement[count_start:(count_start + sRNA_length)])
        if query_seq_fwd in seq_dict and query_seq_fwd in split_alignment_dict:
            split_alignment_dict[query_seq_fwd]+=1
        elif query_seq_fwd in seq_dict and query_seq_fwd\
         not in split_alignment_dict:
            split_alignment_dict[query_seq_fwd]=1
        if query_seq_rvs in seq_dict and query_seq_rvs in split_alignment_dict:
            split_alignment_dict[query_seq_rvs]+=1
        elif query_seq_rvs in seq_dict and query_seq_rvs\
         not in split_alignment_dict:
            split_alignment_dict[query_seq_rvs]=1
        count_start += 1  
    return split_alignment_dict 

def times_read_aligns(split_alignment_dict):
    """
    Dict. of times a read aligns to all references
    {read: times aligned} 
    """
       
    sRNA_align_counts={}
    for aligned_sRNAs in split_alignment_dict.values():
        for aligned_sRNA, count in aligned_sRNAs.iteritems():
            if aligned_sRNA in sRNA_align_counts:
                sRNA_align_counts[aligned_sRNA] += count
            else:
                sRNA_align_counts[aligned_sRNA] = count
    return sRNA_align_counts


def split_reads_for_header(split_align_dict, split_align_count_dict, seq_dict):
    """
    Dict -->Even split read aligned counts, so total reads aligned = read count
    in original seq file
    {header:split_count}
    """
    
    header_split_count = {}
    for header, sRNA_dict in split_align_dict.iteritems():
        header_split_count[header] = 0
        for sRNA in sRNA_dict:
            header_split_count[header]\
             += ((seq_dict[sRNA]/split_align_count_dict[sRNA])\
                 *split_align_dict[header][sRNA])
    return header_split_count


def header_x_y_counts(header_split_count_1, header_split_count_2, refs):
    """
    For each header in reference that has > 0 alignments in 1 file
    
    dict --> {header: (split counts 1:split counts 2)}
    """
    #construct x,y counts for each header
    counts_by_ref = {}
    for header in refs.headers():
        if header in header_split_count_1 and header in header_split_count_2:
            counts_by_ref[header] = (header_split_count_1[header], 
                                     header_split_count_2[header])
        elif header in header_split_count_1 and header \
        not in header_split_count_2:
            counts_by_ref[header] = (header_split_count_1[header], 0)
        elif header not in header_split_count_1 and header in \
        header_split_count_2:
            counts_by_ref[header] = (0, header_split_count_2[header])
    return counts_by_ref 


def CDP_output(counts_by_ref, fileFig, fileName, onscreen, no_csv, seq_name_1, 
               seq_name_2, ref_file, nt, pub):
    """
    Organise csv or pdf output for CDP analysis
    """
    
    if fileFig or onscreen:
        ref_name = ah.single_file_output(ref_file)
        if fileName == "auto":
            fileName = ah.cdp_file_output(seq_name_1, 
                                       seq_name_2, 
                                       ref_name,
                                       nt, 
                                       "pdf")
        pr.cdp_plot(counts_by_ref, 
                            seq_name_1, 
                            seq_name_2,
                            nt,
                            onscreen,
                            fileFig,
                            fileName,
                            pub)
    
    if no_csv:
        out_csv_name = ah.cdp_file_output(seq_name_1, 
                                           seq_name_2, 
                                           ref_name,
                                           nt, 
                                           "csv")
        
        wtf.cdp_output(counts_by_ref, 
                                 seq_name_1, 
                                 seq_name_2,
                                 out_csv_name) 



'''
Small RNA storage class

Stores unique sequence and read count in an internal dictionary.  
Automatic normalisation to RPMR once loaded from file

Created on 1 Apr 2016

@author: steve
'''

from dna import DNA
import time

class SRNA_Seq(object):
    def __init__(self):
        self._internal_dict = {} #internal dictionary for class
        
    def __setitem__(self, sequence, count):
        self._internal_dict[sequence]=count #{sequence:count}
    
    def __getitem__(self, sequence):
        return self._internal_dict[sequence] #get count for sequence
    
    def __iter__(self):
        return self._internal_dict.iteritems() #iterable
    
    def __len__(self):
        return len(self._internal_dict) #number of sequences stored

    def __contains__(self, sequence):
        return sequence in self._internal_dict #true if sequence stored

    def sRNAs(self):
        return self._internal_dict.keys() #returns a list of all sequences

    def counts(self):
        return self._internal_dict.values() #returns a list all counts

    def load_seq_file(self, seq_file, sRNA_max_len_cutoff, min_reads, \
        sRNA_min_len_cutoff):
        """
        load a sequence file in .fasta format.
    
        Calculate RPMR and apply in the function
    
        sRNA_len_cutoff --> only reads  of length <= sRNA_len_cutoff
        accepted (including for RPMR calculation)
        min_reads --> only reads >= min_reads accepted
    
        Input format MUST be:
        >x-y
        seq
        
        Where y is the read count and seq is the processed read sequence
        (x is ignored, but is the read count rank in Fastx toolkit processed
        files)
        
        produce SRNA-Seq object --> sRNA:RPMR
        """
        start = time.clock()

        read_count = 0
        with open(seq_file, 'rU') as loaded_seq:
    
            for line in loaded_seq:
                line=line.strip()
                if line[0] == '>':
                    count = int(line.split('-')[1])
                    next_line = True 
                elif check_sRNA_allowed(count, len(line), 
                                        sRNA_min_len_cutoff, 
                                        sRNA_max_len_cutoff, 
                                        min_reads) and next_line:
                    self._internal_dict[DNA(line)] = count
                    read_count += count
                    next_line = False                
                else:
                    pass
    
        loaded_seq.close()

        # final RPMR - could simplify in future
        for sRNA, count in self._internal_dict.iteritems():
            self._internal_dict[sRNA] = count * (float(1000000) / read_count)
        print "\n{0} load time = {1} seconds for {2} reads".format(seq_file.split('/')[-1], 
                                            str((time.clock() - start)), read_count)
        print "-"*50

    def load_seq_file_arg_list(self, seq_file_arg_list, sRNA_max_len_cutoff, 
                               min_reads, sRNA_min_len_cutoff):
        """
        load seq dict from arg_list in fasta format.
    
        Calculate RPMR and apply in the function
    
        sRNA_len_cutoff --> only reads  of length <= sRNA_len_cutoff
        accepted (including for RPMR calculation)
        min_reads --> only reads >= min_reads accepted
    
        Input format MUST be:
        >x-y
        seq
    
        produce seq_dict --> sRNA:RPMR where RPMR is the average count for 
        that read if the read is present in ALL sequence files
        """
        start = time.clock()

        # read_count_1 = 0
        indv_seq_dict_list=[] #list of individual seq_dics
        indv_seq_dict_list_factor=[] #RPMR for each seq. disc

        for seq_file in seq_file_arg_list:
            single_start=time.clock()
            seq_dict={}
            read_count = 0

            with open(seq_file, 'rU') as loaded_seq:
        
                for line in loaded_seq:
                    line=line.strip()
                    if line[0] == '>':
                        count = int(line.split('-')[1])
                        next_line = True 
                    elif check_sRNA_allowed(count, len(line), 
                                            sRNA_min_len_cutoff, 
                                            sRNA_max_len_cutoff, 
                                            min_reads) and next_line:
                        seq_dict[DNA(line)] = count
                        read_count += count
                        next_line = False                
                    else:
                        pass
        
            loaded_seq.close()

            indv_seq_dict_list.append(seq_dict)
            indv_seq_dict_list_factor.append(float(1000000)/read_count)
            print "\n{0} load time = {1} seconds for {2} reads".format(seq_file.split('/')[-1], 
                                            str((time.clock() - single_start)), read_count)
        for sRNA, count in indv_seq_dict_list[0].iteritems():
            if all(sRNA in d for d in indv_seq_dict_list):
                total_count = 0
                for i in range(len(indv_seq_dict_list)):
                    total_count+=(indv_seq_dict_list[i][sRNA]*indv_seq_dict_list_factor[i])


                self._internal_dict[sRNA] = total_count/len(indv_seq_dict_list)

        print "\nTotal sequence file processing time = "\
         + str((time.clock() - start)) + " seconds\n"
        print "-"*50

def check_sRNA_allowed(count, length, sRNA_min_len_cutoff, sRNA_max_len_cutoff, 
                       min_reads):
    """
    Return True if sRNA is between min and max length, and above min count 
    """
    if count >= min_reads and sRNA_min_len_cutoff <= length <= sRNA_max_len_cutoff:
        return True
    return False


'''
Created on 4 Apr 2016

@author: steve
'''
import operator
from dna import DNA

class Align_sRNA(object):
    
    def __init__(self):
        self._internal_dict = {}
    
    def __setitem__(self, sRNA, alignment):
        self._internal_dict[sRNA]=alignment
    
    def __getitem__(self, sRNA):
        return self._internal_dict[sRNA]
    
    def __iter__(self):
        return self._internal_dict.iteritems()
    
    def __len__(self):
        return len(self._internal_dict)

    def sRNAs(self):
        return self._internal_dict.keys()

    def alignments(self):
        return self._internal_dict.values()     
        
    def align_reads_to_seq(self, seq_dict, ref, sRNA_length):
        """
        
        Aligns reads from a srna_seq object to a ref_seq object
        
        Returns {query seq: [position, count]} 
        """ 
        count_start = 0

        ref_complement = ref.complement()
    
        while count_start < (len(ref) - (sRNA_length - 1)):
            query_seq_fwd = DNA(ref[count_start:(count_start + sRNA_length)])
            query_seq_rvs = DNA(ref_complement[count_start:(count_start + sRNA_length)])
            if query_seq_fwd in seq_dict and query_seq_fwd not in self._internal_dict:
                self._internal_dict[query_seq_fwd] = [
                    [count_start, seq_dict[query_seq_fwd]]]
            elif query_seq_fwd in seq_dict and query_seq_fwd in self._internal_dict:
                self._internal_dict[query_seq_fwd].append(
                    [count_start, seq_dict[query_seq_fwd]])
            if query_seq_rvs in seq_dict and query_seq_rvs not in self._internal_dict:
                self._internal_dict[query_seq_rvs] = [
                    [len(ref)-count_start-1, 0 - seq_dict[query_seq_rvs]]]
            elif query_seq_rvs in seq_dict and query_seq_rvs in self._internal_dict:
                self._internal_dict[query_seq_rvs].append(
                    [len(ref)-count_start-1, 0 - seq_dict[query_seq_rvs]])
            count_start += 1       

    def split(self):
        """
        Splits the alignment count for a read by the number of times it has aligned.
        
        Returns an alignment object
        """
        for alignments in self._internal_dict.values():
            for alignment in alignments:
                alignment[1]/=len(alignments)
        
    
    
    def aln_by_ref_pos(self):
        """
        Create 2 lists - fwd alignment and rvs alignment
        Each contains tuple(pos,count)
        Returned as ordered.
        """
    
        fwd_alignment={}
        rvs_alignment={}
        aln_count = 0
        for alignment in self._internal_dict.itervalues():
            for i in alignment:
                if i[1] > 0:
                    fwd_alignment[i[0]]=i[1] # pos,count
                    aln_count += i[1]
                elif i[1] < 0: 
                    rvs_alignment[i[0]]=i[1] #pos:count
                    
                    aln_count -= i[1]
    
    
        sorted_fwd_alignment = sorted(fwd_alignment.items(), 
                                      key=operator.itemgetter(0))
    
    
        sorted_rvs_alignment = sorted(rvs_alignment.items(), 
                                      key=operator.itemgetter(0))
    #    detect_phase(sorted_fwd_alignment, sorted_rvs_alignment)
        print "\n{0} reads per million reads have aligned\n".format(aln_count)
        print "-"*50
        return [sorted_fwd_alignment, sorted_rvs_alignment, aln_count]

    


'''
Created on 22 Jan 2016

@author: steve
'''
"""
Alignment post-processing module 
"""

import numpy

def fill_in_zeros(fwd_rvs_align_list, ref_len, nt):
    """
    Takes sorted alignment lists (fwd and rvs) as an input
    Produces 3 lists:
        reference_x_axis (every nucleotide index in the ref_seq)
        fwd_alignment_y_axis (+ve y values for each ref_seq index)
        rvs_alignment_y_axis (-ve y values for each ref_seq index)

    """
    sorted_fwd_alignment=fwd_rvs_align_list[0]
    sorted_rvs_alignment=fwd_rvs_align_list[1]

    fwd_alignment_y_axis=[0]*(ref_len)
    revs_alignment_y_axis=[0]*(ref_len)
    
    reference_x_axis = range(0,ref_len)


    for i in sorted_fwd_alignment:
        fwd_alignment_y_axis[i[0]+(nt/2)]=i[1]
    for i in sorted_rvs_alignment:
        revs_alignment_y_axis[i[0]-(nt/2)]=i[1]


    return  reference_x_axis, fwd_alignment_y_axis, revs_alignment_y_axis


def fill_in_zeros_no_offset(fwd_rvs_align_list, ref_len, nt):
    """
    Takes sorted alignment lists (fwd and rvs) as an imput
    Produces 3 lists:
        reference_x_axis (every nucleotid index in the ref_seq)
        fwd_alignment_y_axis (+ve y values for each ref_seq index)
        rvs_alignment_y_axis (-ve y values for each ref_seq index)

    """
    sorted_fwd_alignment=fwd_rvs_align_list[0]
    sorted_rvs_alignment=fwd_rvs_align_list[1]

    fwd_alignment_y_axis=[0]*(ref_len)
    revs_alignment_y_axis=[0]*(ref_len)
    
    reference_x_axis = range(0,ref_len)


    for i in sorted_fwd_alignment:
        fwd_alignment_y_axis[i[0]]=i[1]
    for i in sorted_rvs_alignment:
        revs_alignment_y_axis[i[0]]=i[1]


    return  reference_x_axis, fwd_alignment_y_axis, revs_alignment_y_axis


def calc_alignments_by_strand(fwd_rvs_align_list):
    """
    FILL output
    """

    sorted_fwd_alignment=fwd_rvs_align_list[0]
    sorted_rvs_alignment=fwd_rvs_align_list[1]
    fwd_align_count = 0
    rvs_align_count = 0
    for i in sorted_fwd_alignment:
        fwd_align_count += i[1]
    for i in sorted_rvs_alignment:
        rvs_align_count -= i[1]  
    return fwd_align_count, rvs_align_count




def smooth(x,window_len,window='hamming'):
    """smooth the data using a window with requested size.
    
    This method is based on the convolution of a scaled window with the signal.
    The signal is prepared by introducing reflected copies of the signal 
    (with the window size) in both ends so that transient parts are minimized
    in the begining and end part of the output signal.
    
    input:
        x: the input signal 
        window_len: the dimension of the smoothing window; should be an odd integer
        window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'
            flat window will produce a moving average smoothing.

    output:
        the smoothed signal
        
    example:

    t=linspace(-2,2,0.1)
    x=sin(t)+randn(len(t))*0.1
    y=smooth(x)
    
    see also: 
    
    numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve
    scipy.signal.lfilter
 
    TODO: the window parameter could be the window itself if an array instead of a string
    NOTE: length(output) != length(input), to correct this: return y[(window_len/2-1):-(window_len/2)] instead of just y.
    """

    if x.ndim != 1:
        raise ValueError, "smooth only accepts 1 dimension arrays."

    if x.size < window_len:
        raise ValueError, "Input vector needs to be bigger than window size."


    if window_len<6:
        return x


    if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError, "Window is one of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'"


    s=numpy.r_[x[window_len-1:0:-1],x,x[-1:-window_len:-1]]
    #print(len(s))
    if window == 'flat': #moving average
        w=numpy.ones(window_len,'d')
    else:
        w=eval('numpy.'+window+'(window_len)')

    y=numpy.convolve(w/w.sum(),s,mode='valid')
    return y[(window_len/2-1):-(window_len/2)]

"""End scipy cookbook code
"""




'''
Created on 22 Jan 2016

@author: steve
'''
"""
Analysis module
"""
from ref_seq import Ref_Seq
from srna_seq import SRNA_Seq
import write_to_file
import cdp
import analysis_helper as ah
import den as dn
from termcolor import colored


def single_ref_coverage(seq_file_list, ref_file, nt, smoothWinSize=50, 
    fileFig = False, fileName = 'plot.pdf', min_read_size = 18, 
    max_read_size = 32, min_read_no=1, onscreen = False, no_csv = False, 
    ylim=0, pub=False, split=False):
    """
    Aligns reads from a single read file to a single reference sequence for
    a single sRNA size.
    """
    print colored("-----------------LOADING SEQUENCES----------------",'green')
    seq=SRNA_Seq()
    if len(seq_file_list)==1:
        seq.load_seq_file(seq_file_list[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq.load_seq_file_arg_list(seq_file_list, max_read_size, min_read_no, 
        min_read_size)
    # single_seq_output = ah.single_file_output(seq_file)
    seq_name = ah.single_file_output(seq_file_list[0])
    if len(seq_file_list)>1:
        for i in range(len(seq_file_list)):
            if i == 0:
                pass
            else:
                seq_name+= "_{0}".format(ah.single_file_output(seq_file_list[i]))      
    dn.ref_coverage(seq, seq_name, ref_file, nt, smoothWinSize, fileFig, 
                 fileName, min_read_size, max_read_size, min_read_no, 
                 onscreen, no_csv, ylim, pub, split)


def single_ref_coverage_21_22_24(seq_file_list, ref_file, smoothWinSize=50, 
    fileFig = True, fileName = 'plot.pdf', min_read_size = 18, 
    max_read_size = 32, min_read_no=1, onscreen = True, no_csv=False,
    y_lim=0, pub=False, split=False):
    """
    Align reads from a single seq file to a single reference for 21,22 and 24nt
    """
    print colored("-----------------LOADING SEQUENCES----------------",'green')
    seq=SRNA_Seq()
    if len(seq_file_list)==1:
        seq.load_seq_file(seq_file_list[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq.load_seq_file_arg_list(seq_file_list, max_read_size, min_read_no, 
        min_read_size)
    # single_seq_output = ah.single_file_output(seq_file)
    seq_name = ah.single_file_output(seq_file_list[0])
    if len(seq_file_list)>1:
        for i in range(len(seq_file_list)):
            if i == 0:
                pass
            else:
                seq_name+= "_{0}".format(ah.single_file_output(seq_file_list[i]))    
    dn.coverage_21_22_24(seq, seq_name, ref_file, smoothWinSize, 
    fileFig, fileName, min_read_size, max_read_size, min_read_no,
    onscreen, no_csv,y_lim, pub, split)


def CDP(seq_file_list_1, seq_file_list_2, ref_file, nt, 
    fileFig=False, fileName = 'plot.pdf', 
    min_read_size = 18, max_read_size = 32, min_read_no=1, onscreen = False, 
    no_csv=False, pub=True,processes=4):
    """
    Plots alignment count for each sRNA in ref file as (x,y)
    for 2 seq files.  No splitting of read count
    """  
    print colored("-----------------LOADING SEQUENCES----------------",'green')
    seq_1=SRNA_Seq()
    if len(seq_file_list_1)==1:
        seq_1.load_seq_file(seq_file_list_1[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq_1.load_seq_file_arg_list(seq_file_list_1, max_read_size, min_read_no, 
        min_read_size)

    seq_2=SRNA_Seq()
    if len(seq_file_list_2)==1:
        seq_2.load_seq_file(seq_file_list_2[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq_2.load_seq_file_arg_list(seq_file_list_2, max_read_size, min_read_no, 
        min_read_size)    
    
    #REPEATED CODE - new function required
    seq_name_1 = ah.single_file_output(seq_file_list_1[0])
    if len(seq_file_list_1)>1:
        for i in range(len(seq_file_list_1)):
            if i == 0:
                pass
            else:
                seq_name_1+= "_{0}".format(ah.single_file_output(seq_file_list_1[i]))
    
    seq_name_2 = ah.single_file_output(seq_file_list_2[0])
    if len(seq_file_list_2)>1:
        for i in range(len(seq_file_list_2)):
            if i == 0:
                pass
            else:
                seq_name_2+= "_{0}".format(ah.single_file_output(seq_file_list_2[i]))

    
    cdp.CDP_shared(seq_1, seq_2, seq_name_1, seq_name_2, ref_file, nt,fileFig, 
               fileName, min_read_size, max_read_size, min_read_no, onscreen,
               no_csv, pub, processes)


def CDP_split(seq_file_list_1, seq_file_list_2, ref_file, nt, 
    fileFig=False, fileName = 'plot.pdf', 
    min_read_size = 18, max_read_size = 32, min_read_no=1, onscreen = False, 
    no_csv=False, pub=False, processes=4):

    """
    Plots alignment count for each sRNA in ref file as (x,y)
    for 2 seq files.  Read count split by number of times an sRNA aligns
      
    """  
    print colored("-----------------LOADING SEQUENCES----------------",'green')
    seq_1=SRNA_Seq()
    if len(seq_file_list_1)==1:
        seq_1.load_seq_file(seq_file_list_1[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq_1.load_seq_file_arg_list(seq_file_list_1, max_read_size, min_read_no, 
        min_read_size)

    seq_2=SRNA_Seq()
    if len(seq_file_list_2)==1:
        seq_2.load_seq_file(seq_file_list_2[0], max_read_size, min_read_no, 
        min_read_size)
    else:
        seq_2.load_seq_file_arg_list(seq_file_list_2, max_read_size, min_read_no, 
        min_read_size)   
    
    
    seq_name_1 = ah.single_file_output(seq_file_list_1[0])
    if len(seq_file_list_1)>1:
        for i in range(len(seq_file_list_1)):
            if i == 0:
                pass
            else:
                seq_name_1+= "_{0}".format(ah.single_file_output(seq_file_list_1[i]))
    
    seq_name_2 = ah.single_file_output(seq_file_list_2[0])
    if len(seq_file_list_2)>1:
        for i in range(len(seq_file_list_2)):
            if i == 0:
                pass
            else:
                seq_name_2+= "_{0}".format(ah.single_file_output(seq_file_list_2[i]))
    
    cdp.CDP_split_shared(seq_1, seq_2, seq_name_1, seq_name_2, ref_file, 
                     nt, fileFig, fileName,min_read_size, max_read_size, 
                     min_read_no, onscreen, no_csv, pub, processes)



'''
Created on 31 Mar 2016

@author: steve
'''
from ref_seq import Ref_Seq
import time
from termcolor import colored
from align_srna import Align_sRNA
import analysis_helper as ah
import numpy
import write_to_file as wtf
import post_process as pp
import plot_reads as pr
from multiprocessing import Process, JoinableQueue, Manager

def ref_coverage(seq, seq_output, ref_file, nt, smoothWinSize, fileFig, 
                 fileName, min_read_size, max_read_size, min_read_no, 
                 onscreen, no_csv, ylim, pub, split):
    """
    Align reads of one length to a single reference sequence
    """
    
    ref = Ref_Seq()
    ref.load_ref_file(ref_file)
    
    if len(ref)>1:
        print "\nMultiple reference sequences in file.\n"
        return 
    ref_output = ah.single_file_output(ref_file)
    #this is a hack    
    for header in ref.headers():
        single_ref=ref[header]
    start = time.clock()
    print colored("------------------ALIGNING READS------------------\n",'green') 
    single_alignment = Align_sRNA()
    single_alignment.align_reads_to_seq(seq, single_ref, nt)
    if split is False:
        single_alignment.split()
    single_sorted_alignemts = single_alignment.aln_by_ref_pos()
    if no_csv:
        wtf.csv_output(single_alignment,
                                 nt,
                                 seq_output,
                                 ref_output)   
    if fileFig or onscreen:
        
        graph_processed = pp.fill_in_zeros(single_sorted_alignemts, 
            len(single_ref), nt)
        x_ref = graph_processed[0]
        y_fwd_smoothed = pp.smooth(numpy.array(graph_processed[1]), 
            smoothWinSize, window='blackman')
        y_rvs_smoothed = pp.smooth(numpy.array(graph_processed[2]), 
            smoothWinSize, window='blackman')
        
        if fileName == "auto":
            fileName = ah.ref_seq_nt_output(seq_output, ref_output, nt, "pdf")
                
        pr.den_plot(x_ref, y_fwd_smoothed, y_rvs_smoothed, nt, fileFig, 
            fileName, onscreen, ref_output, ylim, pub)

def coverage_21_22_24(seq, seq_output, ref_file, smoothWinSize, 
    fileFig, fileName, min_read_size, max_read_size, min_read_no,
    onscreen, no_csv,y_lim, pub, split):     
    """
    Align reads of 21,22 and 24 nt to a single reference seq.
    """
    ref = Ref_Seq()
    ref.load_ref_file(ref_file)
    if len(ref)>1:
        print "\nMultiple reference sequences in file.\n"
        return 
    ref_output = ah.single_file_output(ref_file)
    #this is a hack
    print colored("------------------ALIGNING READS------------------\n",'green')     
    for header in ref.headers():
        single_ref=ref[header]    
    combined_21_22_24(seq, seq_output, ref_output, single_ref, smoothWinSize, 
    fileFig, fileName, min_read_size, max_read_size, min_read_no,
    onscreen, no_csv,y_lim, pub, split)    
       
    
def combined_21_22_24(seq, seq_output, ref_output, single_ref, smoothWinSize, 
    fileFig, fileName, min_read_size, max_read_size, min_read_no,
    onscreen, no_csv,y_lim, pub, split):
    """
    Helper function - Align reads of 21,22 and 24 nt to a single reference 
    sequence
    """
    sRNA_lens = [21,22,24]
    work_queue = JoinableQueue()
    processes = []
    mgr=Manager()
    alignments_dict=mgr.dict()

    non_srt_alignments_dict=mgr.dict() #for csv only
    for x in sRNA_lens:
        work_queue.put(x)
    for w in xrange(3):
        p = Process(target=worker, args=(work_queue, seq, single_ref, split, 
                                         alignments_dict, 
                                         non_srt_alignments_dict, 
                                         no_csv))
        p.start()
        processes.append(p)
    
    for p in processes:
        p.join()    
    

    single_sorted_alignemts_21=alignments_dict[21]

    single_sorted_alignemts_22=alignments_dict[22]

    single_sorted_alignemts_24=alignments_dict[24]

    

    if fileFig or onscreen:
    
        graph_processed_21 = pp.fill_in_zeros(alignments_dict[21], 
            len(single_ref),21)
        graph_processed_22 = pp.fill_in_zeros(alignments_dict[22], 
            len(single_ref),22)
        graph_processed_24 = pp.fill_in_zeros(alignments_dict[24], 
            len(single_ref),24)
    
        x_ref = graph_processed_21[0]
        y_fwd_smoothed_21 = pp.smooth(numpy.array(graph_processed_21[1]), 
            smoothWinSize, window='blackman')
        y_rvs_smoothed_21 = pp.smooth(numpy.array(graph_processed_21[2]), 
            smoothWinSize, window='blackman')
        y_fwd_smoothed_22 = pp.smooth(numpy.array(graph_processed_22[1]), 
            smoothWinSize, window='blackman')
        y_rvs_smoothed_22 = pp.smooth(numpy.array(graph_processed_22[2]), 
            smoothWinSize, window='blackman')
        y_fwd_smoothed_24 = pp.smooth(numpy.array(graph_processed_24[1]), 
            smoothWinSize, window='blackman')
        y_rvs_smoothed_24 = pp.smooth(numpy.array(graph_processed_24[2]), 
            smoothWinSize, window='blackman')
    
        if fileName == "auto":
            fileName = ah.ref_seq_output(seq_output, ref_output, "pdf")
    
        pr.den_multi_plot_3(x_ref, y_fwd_smoothed_21, y_rvs_smoothed_21,
        y_fwd_smoothed_22, y_rvs_smoothed_22, y_fwd_smoothed_24, 
        y_rvs_smoothed_24, fileFig, fileName, onscreen, ref_output, y_lim, pub)
    if no_csv:
        wtf.mnt_csv_output(non_srt_alignments_dict[21], 
                           non_srt_alignments_dict[22], 
                           non_srt_alignments_dict[24],
                           seq_output, 
                           ref_output)

def worker(work_queue, seq, single_ref, split, alignments_dict, 
           non_srt_alignments_dict, 
           no_csv):


    try:
        while not work_queue.empty():
            single_alignment = Align_sRNA()
            sRNA_len = work_queue.get()
            single_alignment.align_reads_to_seq(seq, single_ref, sRNA_len)

            if split is False:
                single_alignment.split()
            if no_csv:
                non_srt_alignments_dict[sRNA_len]=single_alignment
            single_sorted_alignemts = single_alignment.aln_by_ref_pos()
            alignments_dict[sRNA_len] = single_sorted_alignemts
    except Exception, e:
        print e
    return True 


     
        
    


'''
Created on 22 Jan 2016

@author: steve
'''
"""
Write results to file (csv format)
"""

import csv

def csv_output(alignment_dict, nt, seq_file_name, header):
    """
    Write to file --> csv
    sRNA,pos,count 
    """
    alignment_list=[]
    
    for sRNA, alignment in alignment_dict:
        for i in alignment:
            alignment_list.append((sRNA,i[0],i[1]))
    alignment_list.sort(key=lambda tup : tup[1])
    with open(header+'_'+seq_file_name+'_'+str(nt)\
              +'.csv', 'wb') as csvfile:
        mycsv = csv.writer(csvfile, delimiter=',')
        mycsv.writerow(['sRNA','5-prime nuc. position', 'Count'])
        for alignment in alignment_list:
            out=[str(alignment[0]),alignment[1]+1,alignment[2]]
            mycsv.writerow(out)
    csvfile.close()


def mnt_csv_output(alignment_dict_21, alignment_dict_22, alignment_dict_24,
                    seq_file_name, header):
    """
    For mnt - write 3 seperate csvs for 21,22,24 nt sRNA lengths
    """
    csv_output(alignment_dict_21, 21, seq_file_name, header)
    csv_output(alignment_dict_22, 22, seq_file_name, header)
    csv_output(alignment_dict_24, 24, seq_file_name, header)        

def cdp_output(counts_by_ref, header1, header2, out_file):
    """
    Write to file --> csv
    sRNA, seq1_count, seq2_count
    """
    results_list=[]
    for header, counts in counts_by_ref.iteritems():
        results_list.append((header, counts[0], counts[1]))
    with open(out_file, 'wb') as csvfile:
        mycsv = csv.writer(csvfile, delimiter=',')
        mycsv.writerow(['',header1, header2])
        for result in results_list:
            out=[result[0],result[1], result[2]]
            mycsv.writerow(out)

    csvfile.close()







'''
Created on 27 May 2016

@author: steve
'''
from setuptools import setup

setup(name='scram',
      version='0.5.5',
      description=' Small Complementary RnA Mapper',
      url='https://github.com/Carroll-Lab/scram',
      author='Stephen Fletcher',
      author_email='s.fletcher@uq.edu.au',
      license='MIT',
      packages=['scram_modules'],
      classifiers=[
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 4 - Beta',
    'Intended Audience :: Science/Research',
    # Indicate who your project is intended for

    'Topic :: Scientific/Engineering :: Bio-Informatics',

    # Pick your license as you wish (should match "license" above)
     'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 2.7'],
      install_requires=['numpy','matplotlib','termcolor'],
      scripts=['scram_modules/scram',
      'scram_modules/analysis.py',
      'scram_modules/align_srna.py',
      'scram_modules/analysis_helper.py',
      'scram_modules/cdp.py',
      'scram_modules/den.py',
      'scram_modules/dna.py',
      'scram_modules/plot_reads.py',
      'scram_modules/post_process.py',
      'scram_modules/ref_seq.py',
      'scram_modules/srna_seq.py',
      'scram_modules/write_to_file.py',
      ],
      zip_safe=False)